;****************************************************************************
; "RUN" proc to resolve problems under W2K, XP
;
; Latest update: Dec 16, 2003
;
; Overall logic:
;   - If NOSHELL is NOT requested, then...
;      - Check for env-vars "windir", "winbootdir" and "COMSPEC"
;      - If we get either of the first 2:
;          - We check for CMD.EXE, and we use it, and continue.
;          - We check for COMMAND.COM, and exit
;          - Still no success, we proceed to the next step.
;      - If we cannot find CMD.EXE, we try for COMMAND.COM, and use it.
;      - If we're up-that-creek, we just execute the RUN, and hope.....
;
;   - If NOSHELL is requested, then NO Command-Interpreter is allowed !!
;
; Typical usage:
;   Ok = Do_Run(False,0,True,False,False,Cmd_Text)
;   If not Ok Then
;      .......
;   EndIf

Proc Do_Run(Do_Big,Sleep_No,Do_NoRefresh,Do_NoRestore,Do_NoShell,Cmd_Text)

Private Env_Vars,w1,w2,w3,w4,Win_Folder,DOS_CmdInt,Temp_Cmd,
        c,t,Ret_Val,d,p,l,x1,x2

   ; Command_Cmd is a GLOBAL variable !!

   If Not IsAssigned(Command_Cmd) Then			;First-Time ?

      SysInfo Environment to Env_Vars

      w1="" w2="" w3="" w4=""
      If IsAssigned(Env_Vars["Windir"])     Then w1 = Env_Vars["Windir"]     EndIf
      If IsAssigned(Env_Vars["WinBootDir"]) Then w2 = Env_Vars["WinBootDir"] EndIf
      If IsAssigned(Env_Vars["SystemRoot"]) Then w3 = Env_Vars["SystemRoot"] EndIf
      If IsAssigned(Env_Vars["COMSPEC"])    Then w4 = Env_Vars["COMSPEC"]    EndIf

      Switch
         Case Not IsBlank(w1): Win_Folder = w1 + "\\"  	;Path-only
         Case Not IsBlank(w2): Win_Folder = w2 + "\\" 	;Path-only
         Case Not IsBlank(w3): Win_Folder = w3 + "\\" 	;Path-only
         Otherwise           : Win_Folder = ""	 
      EndSwitch
      
      Switch
         Case Not IsBlank(w4): DOS_CmdInt = w4 		;ComSpec - Path+Name
         Otherwise           : DOS_CmdInt = ""
      EndSwitch

      SWITCH

         CASE Win_Folder<>"" and ISFILE(Win_Folder+"SYSTEM32\\cmd.exe") :
            Command_Cmd = Win_Folder + "SYSTEM32\\CMD.EXE /c "

         CASE Win_Folder<>"" and ISFILE(Win_Folder+"command.com") :
            Command_Cmd = Win_Folder + "COMMAND.COM /c "

         CASE DOS_CmdInt<>"" and ISFILE(DOS_CmdInt) :
            Command_Cmd = DOS_CmdInt + " /c "

         CASE ISFILE("C:\\command.com") :
            Command_Cmd = "C:\\COMMAND.COM /c "

         Otherwise:
            Command_Cmd = ""

      ENDSWITCH

   EndIf

   ; Maybe NO Cmd-Interpreter this time !!

   c = IIf (Do_NoShell,"",Command_Cmd)
   t = Cmd_Text

   ; If the overall command is too long (over about 125), it will bomb...
   ; Shorten the Commands, if possible, by removing unnecessary Dir-names

   d = " "+Upper(Directory())		;Eg, " C:\AAAA\BBB\CCC\"
   l = Len(d)

   p = 1
   While p > 0
      p = SearchFrom(d,Upper(c),p)	;anywhere in there ?
      If p > 0 Then
         c = Substr(c,1,p-1) + " " + Substr(c,p+l,255)
         p = p + 1
      EndIf
   EndWhile

   p = 1
   While p > 0
      p = SearchFrom(d,Upper(t),p)	;anywhere in there ?
      If p > 0 Then
         t = Substr(t,1,p-1) + " " + Substr(t,p+l,255)
         p = p + 1
      EndIf
   EndWhile

   ; Remove all unwanted spaces

   While Match(c," ..",c) EndWhile	;(leave the end-space !!!)
   While Match(t,".. ",t) EndWhile	;l-trim
   While Match(t," ..",t) EndWhile	;r-trim
   While Match(c,"..  ..",x1,x2)  c = x1 + " " + x2  EndWhile
   While Match(t,"..  ..",x1,x2)  t = x1 + " " + x2  EndWhile

   ; Double up the \ and " chars...

   c = Double_Embeds(c)			;Convert any \ or " chars to \\ or \"
   t = Double_Embeds(t)      		;(These change RETVAL !!)

   Temp_Cmd = "RUN " +
              IIf(Do_Big      ,"BIG "                       ,"") +
              IIf(Sleep_No > 0,"SLEEP "+StrVal(Sleep_No)+" ","") +
              IIf(Do_NoRefresh,"NOREFRESH "                 ,"") +
              IIf(Do_NoRestore,"NORESTORE "                 ,"") +
              IIf(Do_NoShell  ,"NOSHELL "                   ,"") +
              "\"" + c + t + "\""


   ; Maybe the 2 x "\"" could be omitted from the above line, but I'm not
   ; sure if "RUN xxx xxx cmd" can always cope if the double-quotes are
   ; omitted from around cmd. If the 2 x "\"" can always be omitted, then
   ; the 2 strings must NOT be processed by Double_Embeds().

   l = Len(c+t)
   If l > 125 Then
      Message " WARNING: The RUN Command is very long (len = " + 
              StrVal(l) + ") and it may fail ... [Press ESC] "
      Get_ESC()
      Message " Pass this note to IT: >" + Temp_Cmd + "<...[Press ESC] "
      Get_ESC()
      Message ""
   EndIf

   ;; Message Temp_Cmd		;For tracing/debugging
   ;; Get_ESC()

   Execute Temp_Cmd			;Do the "RUN xxx"
   Ret_Val = RetVal			;(save, for the exit)

   If (NOT Do_NoShell) and (Ret_Val > 0) Then
      Message " RUN Command has FAILED!...Do NOT Proceed...Inform IT...[Press ESC] "
      Get_ESC()
      Message " Pass this note to IT: >" + Temp_Cmd + "<...[Press ESC] "
      Get_ESC()
      Message ""
   EndIf

   Return Ret_Val

EndProc

;****************************************************************************
; Double-Embeds !!
;
;   1. A string might contain embedded \ or embedded " chars.
;   2. The EXECUTE command might process a pre-built STRING.
;   3. If the prepared string contains embedded \ or " chars, these must
;      be preceded with a further \, so that they will be reduced to their
;      correct single-chars when finally executed by the EXECUTE...
;
; Eg, Need to run: Run NoRefresh "Dir A:\\*.*"
;      Changed to: Run NoRefresh x
;         Then to: Execute "Run NoRefresh " + x
; That should run OK...no need to Embed any \ chars...
; But, if we also include the original 2 x " chars in line-1,
;     then we get: Execute "Run NoRefresh "\"" + x + "\"""
; and in this case we need to embed the duplicate \-chars into x.

Proc Double_Embeds(x)
Private p

   p=1
   while p>0
      p=SearchFrom("\\",x,p)
      If p>0 Then
         x = SubStr(x,1,p-1) + "\\" + SubStr(x,p,255)
         p = p + 2
      EndIf
   EndWhile

   p=1
   while p>0
      p=SearchFrom("\"",x,p)
      If p>0 Then
         x = SubStr(x,1,p-1) + "\\" + SubStr(x,p,255)
         p = p + 2
      EndIf
   EndWhile

   Return x			;(Returns "X", and puts it into RETVAL also)

EndProc

;****************************************************************************
; Wait until ESC-key is pressed, with a regular nudge...

Proc Get_Esc()
Private x,t1,gap

   Cursor Normal

   While CharWaiting()  x = GetChar()  EndWhile ;clear buffer

   x = 0
   t1 = Ticks()			;86400 secs per day, * 1000
   Beep

   While True
      gap = Int((Ticks() - t1) / 1000)		;beep every ? secs
      If gap < 0 Then gap = gap + 86400 EndIf	;(midnight)
      If gap >= 10 Then	Beep t1 = Ticks() EndIf	;every  10  Secs !!
   
      While CharWaiting()
         x = GetChar()
         If x = 27 Then
            QuitLoop            ;out of inner loop
         EndIf   
      EndWhile   
      
      If x = 27 then
         QuitLoop               ;out of outer loop
      EndIf
   EndWhile
   
   Message ""

   While CharWaiting()  x = GetChar()  EndWhile ;clear buffer again

   Cursor Off

ENDPROC
